# -*- coding: ISO-8859-1 -*-
# Copyright (C) 2003-2005 Juan David Ibez Palomar <jdavid@itaapy.com>
#                    2005 Alexander Fernandez <alex@itaapy.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

# Import from the Standard Library
from time import time
from urllib import quote

# Import from itools
from itools import types
from itools import handlers
from itools.handlers.Text import Text
from itools import i18n
from itools.resources import base, memory
from itools import uri
from itools.zope import get_context
from itools.zope.utils import checkid

# Import from Zope
from OFS import CopySupport

# Import from ikaaro
from exceptions import UserError
import File
from images import Image
from Handler import Handler
from LocaleAware import LocaleAware
from Metadata import Metadata
from VersioningAware import VersioningAware
from WorkflowAware import WorkflowAware
from utils import comeback, reduce_string
from widgets import Breadcrumb, Table



class Folder(Handler, handlers.Folder.Folder):

    #########################################################################
    # Class metadata
    #########################################################################
    class_id = ''
    class_version = '20040625'
    class_title = u'Folder'
    class_description = u'Organize your files and documents with folders.'
    class_icon16 = 'images/Folder16.png'
    class_icon48 = 'images/Folder48.png'


    search_criteria =  ['title', 'text', 'name']


    #########################################################################
    # Aggregation relationship (what a generic folder can contain)
    class_document_types = []

    __fixed_handlers__ = []


    @classmethod
    def register_document_type(cls, handler_class):
        cls.class_document_types.append(handler_class)


    def get_document_types(self):
        return self.class_document_types


    @classmethod
    def new_instance_form(cls):
        namespace = {'class_id': cls.class_id,
                     'class_title': cls.class_title}

        root = get_context().root
        handler = root.get_handler('ui/Folder_new_instance.xml')
        return handler.stl(namespace)


    #######################################################################
    # Traverse
    #######################################################################
    GET__access__ = True
    def GET(self):
        context = get_context()
        request, response = context.request, context.response

        path = request.uri.path
        # Try index
        for name in ['index.xhtml', 'index.html']:
            if self.has_handler(name):
                path = path.resolve2(name)
                response.redirect(path)
                return
        else:
            return Handler.GET(self)


    def _get_handler(self, segment, resource):
        name = segment.name
        # Metadata
        if name.startswith('.'):
            if name.endswith('.metadata'):
                return Metadata(resource)
        # Get the format
        if self.has_handler('.%s.metadata' % name):
            metadata = self.get_handler('.%s.metadata' % name)
            format = metadata.get_property('format')
        else:
            format = resource.get_mimetype()

        return Handler.build_handler(resource, format)


    def _get_handler_names(self):
        names = handlers.Folder.Folder._get_handler_names(self)
        for name in names:
            if not name.startswith('.'):
                name, type, language = types.FileName.decode(name)
                if language is not None:
                    name = types.FileName.encode((name, type, None))
                    names.append(name)

        return names


    def _get_virtual_handler(self, segment):
        name = segment.name

        languages = [ x.split('.')[-1]
                      for x in self.resource.get_resource_names()
                      if x.startswith(name) ]

        if languages:
            # Get the best variant
            context = get_context()

            if context is None:
                language = None
            else:
                request = context.request
                language = request.accept_language.select_language(languages)

            # By default use whatever variant
            # (XXX we need a way to define the default)
            if language is None:
                language = languages[0]
            return self.get_handler('%s.%s' % (name, language))

        return handlers.Folder.Folder._get_virtual_handler(self, segment)


    def on_set_handler(self, segment, handler, format=None, id=None,
                       move=False, **kw):
        from Root import Root

        name = segment.name
        if not name.startswith('.'):
            context = get_context()
            user = context.user
            # Set metadata
            metadata = handler.get_metadata()
            # Pre-process input parameters
            if format is None:
                format = handler.class_id
##            if not isinstance(handler, LocaleAware):
##                kw['dc:language'] = None
            if isinstance(handler, WorkflowAware):
                kw.setdefault('state', handler.workflow.initstate)
            if id is None:
                if self.parent is None:
                    id = '%s/%s' % (name, time())
                else:
                    id = '%s/%s/%s' % (self.abspath[1:], name, time())
            if metadata is None:
                owner = user is not None and user.name or ''
                metadata = Metadata(handler_class=handler.__class__,
                                    format=format, owner=owner, id=id, **kw)
            self.set_handler('.%s.metadata' % name, metadata)

            root = self.get_root()
            if isinstance(root, Root):
                # Index
                handler = self.get_handler(segment)
                if isinstance(handler, Folder):
                    for x, context in handler.traverse2():
                        if x.real_handler is not None:
                            context.skip = True
                        else:
                            if not x.name.startswith('.'):
                                root.index_handler(x)
                else:
                    root.index_handler(handler)
                # Store revision in the archive
                if isinstance(handler, VersioningAware):
                    if move is False:
                        archive = root.get_handler('.archive')
                        # Add folders to archived file
                        path = id.split('/')
                        for i in range(len(path)):
                            x = '/'.join(path[:i+1])
                            if not archive.has_handler(x):
                                archive.set_handler(x, handlers.Folder.Folder())
                        archive.get_handler(id).set_handler(path[-1], handler)


    def on_del_handler(self, segment):
        from Root import Root

        name = segment.name
        if not name.startswith('.'):
            handler = self.get_handler(segment)
            # Unindex
            root = self.get_root()
            if isinstance(root, Root):
                if isinstance(handler, Folder):
                    for x in handler.traverse():
                        root.unindex_handler(x)
                else:
                    root.unindex_handler(handler)
            # Keep database consistency
            if isinstance(handler, LocaleAware):
                handler.remove_translation()

            # Remove metadata
            self.del_handler('.%s.metadata' % name)


    #######################################################################
    # API
    #######################################################################
    def search_handlers(self, path='.', **kw):
        container = self.get_handler(path)

        format = kw.get('format')
        state = kw.get('state')
        for name in container.get_handler_names():
            if not name.startswith('.'):
                handler = container.get_handler(name)
                if isinstance(handler, LocaleAware):
                    if not handler.is_master():
                        continue
                get_property = getattr(handler, 'get_property', lambda x: None)
                if format is None or get_property('format') == format:
                    if state is None:
                        yield handler
                    elif get_property('state') is None:
                        yield handler
                    elif get_property('state') == state:
                        yield handler


    #######################################################################
    # User interface
    #######################################################################
    def get_firstview(self):
        """
        Returns the first allowed object view url, or None if there aren't.
        """
        for name in self.get_views():
            method = self.get_method(name)
            if method is not None:
                session = get_context().session
                browse_name = session.get('browse', 'browse_thumbnails')
                if name == 'browse_thumbnails':
                    if browse_name == 'list':
                        name = 'browse_list'
                    elif browse_name == 'image_gallery':
                        name = 'browse_image'
                elif name == 'browse_list':
                    if browse_name == 'image_gallery':
                        name = 'browse_image'

                return name
        return None

    firstview = property(get_firstview, None, None, "")


    def get_views(self):
        return ['browse_thumbnails', 'new_resource_form', 'edit_metadata_form']


    def get_subviews(self, view):
        if view in ['browse_thumbnails', 'browse_list', 'browse_image']:
            return ['browse_thumbnails', 'browse_list', 'browse_image']
        return []


    #######################################################################
    # Browse
    def _browse_namespace(self, object):
        pass


    def browse_namespace(self, icon_size, sortby='title_or_name', query={}):
        context = get_context()
        request = context.request

        root = context.root
        path_to_root = context.path.get_pathtoroot()
        search_subfolders = False

        # hack for search in a tree, search_subfolder is a path string
        search_subfolders = query.get('search_subfolders')

        if search_subfolders is not None:
            query['search_subfolders'] = ''
        else:
            query['parent_path'] = self.get_abspath()

        results = self.search(**query)

        # if search in subfolders is active we filter on path
        if search_subfolders is not None:
            abspath = self.get_abspath()
            results = [ x for x in results
                        if x.parent_path.startswith(abspath) ]

        # put the metadatas in a dictionary list to be managed with Table
        fields = root.get_catalog_metadata_fields()
        table_content = []
        for result in results:
            line = {}
            for field in fields:
                # put a '' if the brain doesn't have the given field
                line[field] = getattr(result, field, '')
            table_content.append(line)

        # Build the table
        tablename = 'content'
        table = Table(path_to_root, tablename, table_content, sortby=sortby,
                      sortorder='up', batchstart='0', batchsize='20')

        # get the handler for the visibles documents and extracts values
        objects = []
        for line in table.objects:
            abspath = line['abspath']
            document = root.get_handler(abspath)
            if document.is_allowed_to_view():
                resource = document.resource
                line = {}
                name = document.name
                line['abspath'] = abspath
                line['title_or_name'] = document.title_or_name
                line['name'] = str(self.get_pathto(document))
                line['format'] = document.get_property('format')
                line['class_title'] = document.class_title
                line['title'] = document.get_property('dc:title')
                is_file = isinstance(resource, base.File)
                is_folder = isinstance(resource, base.Folder)
                line['is_file'] = is_file
                line['is_folder'] = is_folder
                line['ctime'] = resource.get_ctime()
                line['mtime'] = resource.get_mtime()
                line['atime'] = resource.get_atime()

                # compute size
                line['size'] = ''
                if is_file:
                    bytes = resource.get_size()
                    kbytes = bytes / 1024.0
                    if kbytes >= 1000:
                        mbytes = kbytes / 1024.0
                        line['size'] = '%.01f Mb' % mbytes
                    else:
                        line['size'] = '%.01f Kb' % kbytes
                else:
                    size = len([ x for x in resource.get_resource_names()
                                 if not x.startswith('.') ])
                    line['size'] = self.gettext('%d obs') % size
                line['url'] = '%s/;%s' % (line['name'],
                                          document.get_firstview())
                path_to_icon = document.get_path_to_icon(icon_size,
                                                         from_handler=self)
                if path_to_icon.startswith(';'):
                    path_to_icon = uri.Path('%s/' % name).resolve(path_to_icon)
                line['icon'] = path_to_icon
                line['short_title'] = reduce_string(document.title_or_name,
                                                    12, 40)
                if 'language' in line.keys():
                    language = line['language']
                    if language:
                        language_name = i18n.get_language_name(language)
                        line['language'] = self.gettext(language_name)
                else:
                    line['language'] = ''
                line['mtime'] = document.mtime.strftime('%Y-%m-%d %H:%M')

                line['workflow_state'] = ''
                if hasattr(document, 'workflow_state'):
                    state = document.workflow_state
                    line['workflow_state'] = self.gettext(state)

                # Document details
                line['details'] = '%s-details' % name
                is_image = isinstance(document, Image)
                line['is_image'] = is_image
                if is_image:
                    line['image_preview'] = '%s/;icon48?width=200&height=160' % name
                line['onclick'] = "hide_details('%s-details')" % name
                # Objects that should not be removed/renamed/etc
                line['checkbox'] = name not in self.__fixed_handlers__
                #
                self._browse_namespace(line)
                objects.append(line)
                
        table.objects = objects

        # Build namespace
        namespace = {}
        namespace['table'] = table
        namespace['batch'] = table.batch_control()
        # Paste?
        try:
            cp = request.cookies['__cp']
            cp = CopySupport._cb_decode(cp)
        except:
            namespace['paste'] = False
        else:
            namespace['paste'] = True
        return namespace


    browse_thumbnails__access__ = Handler.is_allowed_to_view
    browse_thumbnails__label__ = u'View'
    browse_thumbnails__sublabel__ = u'As Icons'
    def browse_thumbnails(self):
        context = get_context()
        context.session['browse'] = 'thumb'

        parent_path = self.get_abspath()
        if parent_path in ('', None):
            parent_path = '/'
        query = {'parent_path' : parent_path}
        namespace = self.browse_namespace(48, query=query)

        handler = self.get_handler('/ui/Folder_browse_thumbnails.xml')
        return handler.stl(namespace)


    browse_list__access__ = Handler.is_allowed_to_view
    browse_list__label__ = u'View'
    browse_list__sublabel__ = u'As List'
    def browse_list(self):
        context = get_context()
        request = context.request
        form = request.form

        search_subfolders = form.get('search_subfolders')
        search_value = form.get('search_value', '').strip()

        if search_subfolders and not search_value:
            comeback('browse_list', 'Please put a value for your search '
                     'criteria if you include subfolders')
        else:
            selected_criteria = form.get('search_criteria')

            context.session['browse'] = 'list'

            query = {}

            if search_value:
                query[selected_criteria] = search_value
            if search_subfolders is not None:
                query['search_subfolders'] = search_subfolders

            namespace = self.browse_namespace(16, query=query)

            namespace['search_value'] = search_value
            namespace['search_subfolders'] = search_subfolders
            namespace['self_path'] = self.get_abspath()

            search_criteria = [
                {'id': x,
                 'title': self.gettext(x.capitalize().replace('_', ' ')),
                 'selected': x == selected_criteria or None}
                for x in self.get_search_criteria() ]
            namespace['search_criteria'] = search_criteria

            handler = self.get_handler('/ui/Folder_browse_list.xml')
            return handler.stl(namespace)


    browse_image__access__ = Handler.is_allowed_to_view
    browse_image__label__ = u'View'
    browse_image__sublabel__ = u'As Image Gallery'
    def browse_image(self, selected_image=None, **kw):
        context = get_context()
        request = context.request
        
        parent_path = self.get_abspath()
        if parent_path in ('', None):
            parent_path = '/'
        query = {'parent_path' : parent_path}

        namespace = self.browse_namespace(48, query=query)
        # Select an image by default
        table = namespace['table']
        for object in table.objects:
            name = object['name']
            handler = self.get_handler(name)
            if isinstance(handler, Image): 
                if selected_image is None:
                    selected_image = name
                object['url'] = request.build_url(preserve=[table.name],
                                                  selected_image=name)
                object['css_id'] = (selected_image == name) and 'selected'
##                object['icon'] = '%s?height=64&width=64' % object['icon']
            else:
                object['css_id'] = None
        # Selected image
        if selected_image is None:
            namespace['selected'] = None
        else:
            image = self.get_handler(selected_image)
            selected = {}
            selected['name'] = image.name
            selected['url'] = '%s/;%s' % (image.name, image.get_firstview())
            selected['preview'] = '%s/;icon48?height=320&width=320' \
                                  % image.name
            selected['width'] = image.get_width()
            selected['height'] = image.get_height()
            selected['format'] = image.metadata.get_property('format')
            selected['lang'] = image.metadata.get_property('dc:language')
            namespace['selected'] = selected

        handler = self.get_handler('/ui/Folder_browse_image.xml')
        return handler.stl(namespace)


    remove__access__ = Handler.is_authenticated
    def remove(self, ids=[], **kw):
        if not ids:
            raise UserError, self.gettext('No objects selected.')

        removed = []
        not_allowed = []

        context = get_context()
        for name in ids:
            handler = self.get_handler(name)
            if handler.is_allowed_to_remove():
##                # Remove versions
##                metadata = handler.metadata
##                for lang in metadata.get_translations():
##                    trans = metadata.get_translation(lang)
##                    if self.has_handler(trans):
##                        self.del_handler(trans)
                # Remove handler
                self.del_handler(name)
                #
                removed.append(name)
            else:
                not_allowed.append(name)

        message = self.gettext('Objects removed: %s.') % ','.join(removed)
        view = self.get_firstview()
        comeback(view, message)


    rename_form__access__ = Handler.is_authenticated
    def rename_form(self, ids=[], **kw):
        # Filter names which the authenticated user is not allowed to move
        handlers = [ self.get_handler(x) for x in ids ]
        names = [ x.name for x in handlers if x.is_allowed_to_move() ]

        # Check input data
        if not names:
            raise UserError, self.gettext('No objects selected.')

        # XXX Hack to get rename working. The current user interface
        # forces the rename_form to be called as a form action, hence
        # with the POST method, but is should be a GET method. Maybe
        # it will be solved after the needed folder browse overhaul.
        context = get_context()
        request, response = context.request, context.response
        if request.environ.get('REQUEST_METHOD') == 'POST':
            ids_list =  '&'.join([ 'ids:list=%s' % x for x in names ])
            response.redirect(';rename_form?%s' % ids_list)

        # Build the namespace
        namespace = {}
        namespace['names'] = names

        # Process the template
        handler = self.get_handler('/ui/Folder_rename.xml')
        return handler.stl(namespace)


    rename__access__ = Handler.is_authenticated
    def rename(self, ids, new_ids, **kw):
        # Process input data
        for i, old_name in enumerate(ids):
            new_name = new_ids[i]
            new_name = checkid(new_name)
            if new_name is None:
                # Invalid name
                raise UserError, self.gettext('The document name contains illegal characters, choose another one.')

            if new_name != old_name:
                handler = self.get_handler(old_name)
                handler_metadata = handler.get_metadata()

                if isinstance(handler, LocaleAware):
                    is_master = handler.is_master()
                    if not is_master:
                        master = handler.get_master_handler()

                # XXX itools should provide an API to copy and move handlers
                self.set_handler(new_name, handler, move=True)
                self.del_handler('.%s.metadata' % new_name)
                self.set_handler('.%s.metadata' % new_name, handler_metadata)

                self.del_handler(old_name)
                # Update isVersionOf/hasVersion
                handler = self.get_handler(new_name)
                if isinstance(handler, LocaleAware):
                    if is_master:
                        translations = getattr(handler.metadata.properties,
                                               'hasVersion', {})
                        for language, name in translations.items():
                            translation = handler.parent.get_handler(name)
                            translation.set_property('isVersionOf', new_name)
                    else:
                        master.set_property('hasVersion', new_name,
                                            language=handler.get_language())

        message = self.gettext('Objects renamed.')
        comeback(self.get_firstview(), message)


    copy__access__ = Handler.is_authenticated
    def copy(self, ids=[], **kw):
        # Filter names which the authenticated user is not allowed to copy
        handlers = [ self.get_handler(x) for x in ids ]
        names = [ x.name for x in handlers if x.is_allowed_to_copy() ]

        if not names:
            raise UserError, self.gettext('No objects selected.')

        context = get_context()
        request, response = context.request, context.response
        path = str(context.path[1:])
        cp = (False, [ '%s/%s' % (path, x) for x in names ])
        cp = CopySupport._cb_encode(cp)
        path = CopySupport.cookie_path(request)
        response.set_cookie('__cp', cp, path=path)

        message = self.gettext('Objects copied.')
        view = self.get_firstview()
        comeback(view, message)


    cut__access__ = Handler.is_authenticated
    def cut(self, ids=[], **kw):
        # Filter names which the authenticated user is not allowed to move
        handlers = [ self.get_handler(x) for x in ids ]
        names = [ x.name for x in handlers if x.is_allowed_to_move() ]

        if not names:
            raise UserError, self.gettext('No objects selected.')

        context = get_context()
        request, response = context.request, context.response
        path = str(context.path[1:])
        cp = (True, [ '%s/%s' % (path, x) for x in names ])
        cp = CopySupport._cb_encode(cp)
        path = CopySupport.cookie_path(request)
        response.set_cookie('__cp', cp, path=path)

        message = self.gettext('Objects cut.')
        view = self.get_firstview()
        comeback(view, message)


    paste__access__ = Handler.is_allowed_to_add
    def paste(self):
        context = get_context()

        cp = context.request.cookies.get('__cp', None)
        if cp is not None:
            root = context.root
            allowed_types = tuple(self.get_document_types())
            cut, paths = CopySupport._cb_decode(cp)
            for path in paths:
                handler = root.get_handler(path)
                if isinstance(handler, allowed_types):
                    name = handler.name
                    # Find a non used name
                    # XXX ROBLES To be tested carefully and optimized
                    while self.has_handler(name):
                        name = name.split('.')
                        id = name[0].split('_')
                        index = id[-1]
                        try:   # tests if id ends with a number
                            index = int(index)
                        except ValueError:
                            id.append('copy_1') 
                        else:
                            try:  # tests if the pattern is '_copy_x'
                               if id[-2] == 'copy':
                                  index = str(index + 1) # increment index
                                  id[-1] = index
                               else:
                                  id.append('copy_1') 
                            except IndexError:
                               id.append('copy_1') 
                            else:
                               pass
                        id = '_'.join(id)
                        name[0] = id
                        name = '.'.join(name)
                    # Unicode is not a valid Zope id
                    name = str(name)
                    # Add it here
                    if cut is True:
                        archive_id = handler.get_archive_id()
                        self.set_handler(name, handler, id=archive_id,
                                         move=True)
                        # Remove original
                        container = handler.parent
                        container.del_handler(name)
                    else:
                        self.set_handler(name, handler)
                        # Fix metadata properties
                        handler = self.get_handler(name)
                        metadata = handler.metadata
                        # Fix state
                        if isinstance(handler, WorkflowAware):
                            metadata.set_property('state', 'private')
                        # Fix owner
                        metadata.set_property('owner', context.user.name)
                        # Fix hasVersion
                        if metadata.has_property('hasVersion'):
                            metadata.del_property('hasVersion')
                        # Fix isVersionOf
                        if metadata.has_property('isVersionOf'):
                            metadata.del_property('isVersionOf')

        message = self.gettext('Objects pasted.')
        view = self.get_firstview()
        comeback(view, message)


    #######################################################################
    # Browse / Translate
    translate_form__access__ = Handler.is_authenticated
    def translate_form(self, ids=[], **kw):
        context = get_context()
        request, response = context.request, context.response

        if not ids:
            raise UserError, self.gettext('No objects selected')

        # Check input data
        site_root = self.get_site_root()
        content_languages = site_root.get_property('ikaaro:website_languages')
        handlers = [ self.get_handler(x) for x in ids ]
        handlers = [ x for x in handlers if isinstance(x, LocaleAware) ]
        handlers = [ x for x in handlers if x.is_allowed_to_translate() ]
        handlers = [ x for x in handlers
                     if [ y for y in content_languages
                          if y not in x.get_available_languages() ] ]
        names = [ x.name for x in handlers ]

        if not names:
            raise UserError, \
                  self.gettext('The selected objects can not be translated.')

        # XXX Hack to get translate working. The current user interface
        # forces the translate_form to be called as a form action, hence
        # with the POST method, but is should be a GET method. Maybe
        # it will be solved after the needed folder browse overhaul.
        if request.environ.get('REQUEST_METHOD') == 'POST':
            ids_list =  '&'.join([ 'ids:list=%s' % x for x in names ])
            response.redirect(';translate_form?%s' % ids_list)

        # Build the namespace
        namespace = {}
        # Documents
        documents = []
        for handler in handlers:
            document = {'name': handler.name,
                        'title_or_name': handler.title_or_name}
            languages = [ x for x in content_languages
                          if x not in handler.get_available_languages() ]
            document['languages'] = [
                {'code': x,
                 'name': self.gettext(i18n.get_language_name(x))}
                for x in languages ]
            documents.append(document)
        namespace['documents'] = documents

        # Process the template
        handler = self.get_handler('/ui/Folder_translate.xml')
        return handler.stl(namespace)


    translate__access__ = Handler.is_authenticated
    def translate(self, names, languages, **kw):
        context = get_context()
        request = context.request

        for i in range(len(names)):
            name, type, language = types.FileName.decode(names[i])
            language = languages[i]
            trans_name = types.FileName.encode((name, type, language))
            name = names[i]
            # Get the original handler and its metadata
            handler = self.get_handler(name)
            metadata = handler.get_metadata()
            # Add the handler
            handler_class = handler.__class__
            trans_handler = handler_class()
            self.set_handler(trans_name, trans_handler,
                             **{'dc:language': language})
            # Fix the original metadata (hasVersion)
            metadata.set_property('hasVersion', trans_name, language=language)
            # Fix the translation metadata (isVersionOf)
            trans_handler = self.get_handler(trans_name)
            trans_metadata = trans_handler.get_metadata()
            trans_metadata.set_property('isVersionOf', name)

        message = self.gettext('Document translations created.')
        comeback(self.get_firstview(), message)


    #######################################################################
    # Add / New Resource
    new_resource_form__access__ = Handler.is_authenticated
    new_resource_form__label__ = u'Add'
    new_resource_form__sublabel__ = u'New Resource'
    def new_resource_form(self, type=None, **kw):
        if type is None:
            # Build the namespace
            namespace = {}

            rows = []
            for handler_class in self.get_document_types():
                # Get the row
                if not rows or rows[-1]['right'] is not None:
                    rows.append({'left': None, 'right': None})
                row = rows[-1]
                # Get the column namespace
                if row['left'] is None:
                    ns = row['left'] = {}
                else:
                    ns = row['right'] = {}
                # Fill the column
                format = quote(handler_class.class_id)
                path_to_icon = '%sui/%s' % (self.get_pathtoroot(),
                                            handler_class.class_icon48)
                ns['format'] = format
                ns['label'] = handler_class.gettext(handler_class.class_title)
                description = handler_class.class_description
                ns['description'] = handler_class.gettext(description)
                ns['url'] = ';new_resource_form?type=' + format
                ns['path_to_icon'] = path_to_icon
            namespace['rows'] = rows

            handler = self.get_handler('/ui/Folder_new_resource.xml')
            return handler.stl(namespace)
        else:
            handler_class = self.get_handler_class(type)
            return handler_class.new_instance_form()


    new_resource__access__ = Handler.is_authenticated
    def new_resource(self, class_id, name, **kw):
        context = get_context()
        request = context.request

        # Check for errors
        name = name.strip()
        if not name:
            # Empty name
            raise UserError, self.gettext('The name must be entered')

        name = checkid(name)
        if name is None:
            raise UserError, self.gettext('The document name contains illegal characters, choose another one.')

        if self.has_handler(name):
            raise UserError, self.gettext('There is already another object with this name')

        # Build the handler
        handler_class = self.get_handler_class(class_id)
        handler = handler_class.new_instance(**kw)

        # Build the name
        name = types.FileName.encode((name, handler.class_extension,
                                      kw.get('dc:language')))
        # Calculate title
        root = self.get_root()
        languages = root.get_property('ikaaro:website_languages')
        default_language = languages[0]
        kw['dc:title'] = {default_language: kw['dc:title']}
        # Add the handler
        self.set_handler(name, handler, **kw)

        message = self.gettext('New resource added')
        comeback(self.get_firstview(), message)


    browse_dir__access__ = Handler.is_authenticated
    def browse_dir(self, id=None):
        namespace = {}
        namespace['bc'] = Breadcrumb(filter_type=File.File, start=self)

        # Avoid general template
        response = get_context().response
        response.set_header('Content-Type', 'text/html; charset=UTF-8')

        handler = self.get_handler('/ui/Folder_browsedir.xml')
        return handler.stl(namespace)


    #######################################################################
    # Add / Upload File
    upload_file__access__ = Handler.is_authenticated
    def upload_file(self, file, **kw):
        name = file.filename.split('\\')[-1]

        # Build a memory resource
        data = file.read()
        resource = memory.File(data, name=name)
        mimetype = resource.get_mimetype()

        # Guess the language if it is not included in the filename
        if mimetype.startswith('text/'):
            name, type, language = types.FileName.decode(name)
            if language is None:
                encoding = Text.guess_encoding(data)
                data = unicode(data, encoding)
                language = i18n.oracle.guess_language(data)

            name = types.FileName.encode((name, type, language))

        # Process input data
        name = name.strip()
        if not name:
            # Empty name
            raise UserError, self.gettext('The file must be entered')

        name = checkid(name)
        if name is None:
            # Invalid name
            raise UserError, self.gettext('The document name contains illegal characters, choose another one.')

        if self.has_handler(name):
            # Name already used
            raise UserError, self.gettext('There is already another resource with this name.')

        # Build the handler
        handler = Handler.build_handler(resource, mimetype)

        # Set the handler
        self.set_handler(name, handler, format=mimetype)

        # Come back
        message = self.gettext('File uploaded.')
        comeback(self.get_firstview(), message)


    #######################################################################
    # Search
    def get_search_criteria(self):
        """Return the criteria as a list of dictionnary
        like [{'id': criteria_id, 'title' : criteria_title},...]
        """
        return self.search_criteria


    def search(self, **kw):
        context = get_context()
        root = context.root
        catalog = root.get_handler('.catalog')
        result = catalog.search(**kw)
        return result


Handler.register_handler_class(Folder)
